#ifndef _OBJECT_HPP_
#define _OBJECT_HPP_

#include <list>
#include <vector>

namespace histodet {


/**
 * Description of an object detected in an image.
 * It can include more or less information.
 */
struct Object
{
	/**
	* Indices of values in the vector
	* @var ltPosU ltPosV : Position in the image
	* @var ltSize : Size in pixels
	* @var ltHeading : Orientation in radian wrt x axis (positive up)
	* @var ltAttitudePitch ltAttitudeRoll : attitude angles in radian
	* @var LocPosSize : size of the vector
	*/
	enum LocPos { lpPosU = 0, lpPosV, lpSize, lpHeading, lpAttitudePitch, lpAttitudeRoll, LocPosSize };

	unsigned type; ///< type of object
	double loc    [LocPosSize]; ///< loc data: (u,v,s,w,p,r)
	double loc_std[LocPosSize]; ///< standard deviations of loc data
	bool   mask   [LocPosSize]; ///< mask to know if corresponding value in loc is valid or not
	double score; ///< detection score between 0 and 1 (1 is best)

	/**
	 * Constructor
	 * @param type type of object
	 */
	Object(unsigned type): type(type)
		{ for(int i = 0; i < LocPosSize; ++i) mask[i] = false; }

	/**
	 * Set a value
	 * @param l the label of the value
	 * @param val the value
	 * @param std the standard deviation of the value
	 */
	inline void set(LocPos l, double val, double std) { mask[l] = true; loc[l] = val; loc_std[l] = std; }


	//private: private stuff
	double raw_score; ///< detection raw score depending on the refiner
	bool operator<(Object const & obj) const ///< used for sorting, best is largest score
	{
		return score > obj.score;
	}

	
};



/**
 * List of objects
 */
typedef std::list<Object> ObjectList;



} // namespace histodet



#endif
