#ifndef _REFINER_ABSTRACT_HPP_
#define _REFINER_ABSTRACT_HPP_

namespace histodet {

	class RefinerAbstract
	{
		public:
			/**
			 * Refinement function that takes an object already extracted to
			 * assert it is the wanted object and refine its
			 * position, orientation and scale
			 * @param object the object with estimated position and size
			 * @param image the whole image
			 * @return whether the object was confirmed or not (and object contains a valid description)
			 */
			virtual bool refine(Object &object, cv::Mat const& image, double &correl_score) = 0;
			/**
			 * Refinement function that takes candidates pixels in a whole image
			 * and looks for the wanted object among them, as well as refines its
			 * position, orientation and scale
			 */
			virtual bool refine(Object &object, cv::Mat const& image, cv::Mat const& detImage) {}; // = 0

		protected:

			cv::Mat extractPatch(Object &object, cv::Mat const& image, cv::Point & offset)
			{
				const double nsigma = 1.0;

				if (!object.mask[Object::lpPosU] || !object.mask[Object::lpPosU] || !object.mask[Object::lpSize])
					return image;

				cv::Rect rect;
				rect.width  = 2*nsigma*object.loc_std[Object::lpPosU] + object.loc[Object::lpSize] + nsigma*object.loc_std[Object::lpSize];
				rect.height = 2*nsigma*object.loc_std[Object::lpPosV] + object.loc[Object::lpSize] + nsigma*object.loc_std[Object::lpSize];
				rect.x = object.loc[Object::lpPosU] - rect.width  / 2;
				rect.y = object.loc[Object::lpPosV] - rect.height / 2;
				if (rect.x < 0) rect.x = 0;
				if (rect.y < 0) rect.y = 0;
				if (rect.x + rect.width  > image.cols) rect.width  = image.cols - rect.x;
				if (rect.y + rect.height > image.rows) rect.height = image.rows - rect.y;

// std::cout << "extractPatch: object located at " << object.loc[Object::lpPosU] << "," << object.loc[Object::lpPosV] << " (sigma " << object.loc_std[Object::lpPosU] << "," << object.loc_std[Object::lpPosV] << ") gives roi (" << rect.x << "," << rect.y << "," << rect.width << "," << rect.height << ")" << std::endl;

				offset.x = rect.x; offset.y = rect.y;
				return cv::Mat(image, rect);
			}

	};


}

#endif